@@ -244,6 +244,34 @@ catalogs__gth_browser_update_sensitivity_cb (GthBrowser *browser)
 	action = gtk_action_group_get_action (data->actions, "Edit_RemoveFromCatalog");
 	sensitive = (n_selected > 0) && GTH_IS_FILE_SOURCE_CATALOGS (gth_browser_get_location_source (browser));
 	g_object_set (action, "sensitive", sensitive, NULL);
+
+	action = gtk_action_group_get_action (data->actions, "Go_FileContainer");
+	sensitive = (n_selected == 1);
+	g_object_set (action, "sensitive", sensitive, NULL);
+}
+
+
+GFile *
+catalogs__command_line_files_cb (GList *files)
+{
+	GFile      *file;
+	GthCatalog *catalog;
+	GList      *scan;
+
+	if (g_list_length (files) <= 1)
+		return NULL;
+
+	file = _g_file_new_for_display_name ("catalog:///", _("Command Line"), ".catalog");
+	catalog = gth_catalog_new ();
+	gth_catalog_set_file (catalog, file);
+	gth_catalog_set_name (catalog, _("Command Line"));
+	for (scan = files; scan; scan = scan->next)
+		gth_catalog_insert_file (catalog, (GFile *) scan->data, -1);
+	gth_catalog_save (catalog);
+
+	g_object_unref (catalog);
+
+	return file;
 }
 
 
