@@ -53,6 +53,8 @@ gth_catalog_finalize (GObject *object)
 {
 	GthCatalog *catalog = GTH_CATALOG (object);
 
+	g_value_hash_unref (catalog->attributes);
+
 	if (catalog->priv != NULL) {
 		if (catalog->priv->file != NULL)
 			g_object_unref (catalog->priv->file);
@@ -80,7 +82,7 @@ base_create_root (GthCatalog  *catalog,
 
 
 static void
-base_read_from_doc (GthCatalog  *catalog,
+base_read_from_doc (GthCatalog *catalog,
 		    DomElement *root)
 {
 	GList      *file_list;
@@ -111,6 +113,8 @@ base_read_from_doc (GthCatalog  *catalog,
 	}
 	gth_catalog_set_file_list (catalog, file_list);
 
+	gth_hook_invoke ("gth-catalog-read-from-doc", catalog, root);
+
 	_g_object_list_unref (file_list);
 }
 
@@ -219,6 +223,8 @@ base_write_to_doc (GthCatalog  *catalog,
 			g_free (uri);
 		}
 	}
+
+	gth_hook_invoke ("gth-catalog-write-to-doc", catalog, doc, root);
 }
 
 
@@ -241,6 +247,7 @@ gth_catalog_class_init (GthCatalogClass *class)
 static void
 gth_catalog_init (GthCatalog *catalog)
 {
+	catalog->attributes = g_value_hash_new ();
 	catalog->priv = g_new0 (GthCatalogPrivate, 1);
 	catalog->priv->date_time = gth_datetime_new ();
 	catalog->priv->file_hash = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal, NULL, NULL);
@@ -561,16 +568,16 @@ catalog_file_info_ready_cb (GObject      *source_object,
 
 
 static void
-list__catalog_buffer_ready_cb (void     *buffer,
-			       gsize     count,
-			       GError   *error,
-			       gpointer  user_data)
+list__catalog_buffer_ready_cb (void     **buffer,
+			       gsize      count,
+			       GError    *error,
+			       gpointer   user_data)
 {
 	ListData   *list_data = user_data;
 	GthCatalog *catalog = list_data->catalog;
 
-	if ((error == NULL) && (buffer != NULL)) {
-		gth_catalog_load_from_data (catalog, buffer,  count, &error);
+	if ((error == NULL) && (*buffer != NULL)) {
+		gth_catalog_load_from_data (catalog, *buffer,  count, &error);
 		if (error != NULL) {
 			gth_catalog_list_done (list_data, error);
 			return;
@@ -764,6 +771,8 @@ gth_catalog_update_metadata (GthCatalog  *catalog,
 				    file_data->info,
 				    catalog->priv->name,
 				    catalog->priv->date_time);
+
+	gth_hook_invoke ("gth-catalog-write-metadata", catalog, file_data);
 }
 
 
@@ -1027,18 +1036,18 @@ typedef struct {
 
 
 static void
-load__catalog_buffer_ready_cb (void     *buffer,
-			       gsize     count,
-			       GError   *error,
-			       gpointer  user_data)
+load__catalog_buffer_ready_cb (void     **buffer,
+			       gsize      count,
+			       GError    *error,
+			       gpointer   user_data)
 {
 	LoadData   *load_data = user_data;
 	GthCatalog *catalog = NULL;
 
 	if (error == NULL) {
-		catalog = gth_hook_invoke_get ("gth-catalog-load-from-data", buffer);
+		catalog = gth_hook_invoke_get ("gth-catalog-load-from-data", *buffer);
 		if (catalog != NULL)
-			gth_catalog_load_from_data (catalog, buffer, count, &error);
+			gth_catalog_load_from_data (catalog, *buffer, count, &error);
 	}
 
 	load_data->ready_func (G_OBJECT (catalog), error, load_data->user_data);
@@ -1130,6 +1139,7 @@ gth_catalog_save (GthCatalog *catalog)
 
 	file = gth_catalog_get_file (catalog);
 	gio_file = gth_catalog_file_to_gio_file (file);
+
 	gio_parent = g_file_get_parent (gio_file);
 	g_file_make_directory_with_parents (gio_parent, NULL, NULL);
 	data = gth_catalog_to_data (catalog, &size);
