@@ -161,7 +161,7 @@ get_attribute_value (GthFileData *file_data,
 	char      *attribute = NULL;
 	char      *value = NULL;
 
-	re = g_regex_new ("%attr\\{([^}]+)\\}", 0, 0, NULL);
+	re = g_regex_new ("%A\\{([^}]+)\\}", 0, 0, NULL);
 	a = g_regex_split (re, match, 0);
 	if (g_strv_length (a) >= 2)
 		attribute = g_strstrip (a[1]);
@@ -194,8 +194,19 @@ template_eval_cb (const GMatchInfo *info,
 	char         *match;
 
 	match = g_match_info_fetch (info, 0);
-	if (strcmp (match, "%F") == 0) {
-		r = g_file_get_basename (template_data->file_data->file);
+
+	if (strncmp (match, "#", 1) == 0) {
+		char *format;
+
+		format = g_strdup_printf ("%%0%" G_GSIZE_FORMAT "d", strlen (match));
+		r = g_strdup_printf (format, template_data->n);
+
+		g_free (format);
+	}
+	else if (strncmp (match, "%A", 2) == 0) {
+		r = get_attribute_value (template_data->file_data, match);
+		/*if (r == NULL)
+			*template_data->error = g_error_new_literal (GTH_TASK_ERROR, GTH_TASK_ERROR_FAILED, _("Malformed template"));*/
 	}
 	else if (strcmp (match, "%E") == 0) {
 		char *uri;
@@ -205,21 +216,43 @@ template_eval_cb (const GMatchInfo *info,
 
 		g_free (uri);
 	}
+	else if (strcmp (match, "%F") == 0) {
+		r = g_file_get_basename (template_data->file_data->file);
+	}
 	else if (strcmp (match, "%N") == 0) {
 		r = get_original_enum (template_data->file_data, match);
 	}
-	else if (strncmp (match, "%attr", 5) == 0) {
-		r = get_attribute_value (template_data->file_data, match);
-		/*if (r == NULL)
-			*template_data->error = g_error_new_literal (GTH_TASK_ERROR, GTH_TASK_ERROR_FAILED, _("Malformed template"));*/
-	}
-	else if (strncmp (match, "#", 1) == 0) {
-		char *format;
+	else if ((strncmp (match, "%D", 2) == 0) || (strncmp (match, "%M", 2) == 0)) {
+		gboolean value_available = FALSE;
+		GTimeVal timeval;
 
-		format = g_strdup_printf ("%%0%" G_GSIZE_FORMAT "d", strlen (match));
-		r = g_strdup_printf (format, template_data->n);
+		if (strncmp (match, "%D", 2) == 0) {
+			value_available = gth_file_data_get_digitalization_time (template_data->file_data, &timeval);
+		}
+		else if (strncmp (match, "%M", 2) == 0) {
+			timeval = *gth_file_data_get_modification_time (template_data->file_data);
+			value_available = TRUE;
+		}
 
-		g_free (format);
+		if (value_available) {
+			GRegex  *re;
+			char   **a;
+			char    *format = NULL;
+
+			/* Get the date format */
+
+			re = g_regex_new ("%[A-Z]\\{([^}]+)\\}", 0, 0, NULL);
+			a = g_regex_split (re, match, 0);
+			if (g_strv_length (a) >= 2)
+				format = g_strstrip (a[1]);
+			if ((format == NULL) || (*format == '\0'))
+				format = "%Y-%m-%d";
+
+			r = _g_time_val_strftime (&timeval, format);
+
+			g_strfreev (a);
+			g_regex_unref (re);
+		}
 	}
 
 	if (r != NULL)
@@ -274,7 +307,7 @@ dlg_rename_series_update_preview (DialogData *data)
 	template_data->error = &error;
 	template_data->n = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (GET_WIDGET ("start_at_spinbutton")));
 	template_data->template = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("template_entry")));
-	re = g_regex_new ("#+|%F|%E|%N|%attr\\{[^}]+\\}", 0, 0, NULL);
+	re = g_regex_new ("#+|%[ADEFMN](\\{[^}]+\\})?", 0, 0, NULL);
 	for (scan = data->new_file_list; scan; scan = scan->next) {
 		char *new_name;
 		char *new_name2;
@@ -301,6 +334,7 @@ dlg_rename_series_update_preview (DialogData *data)
 
 		g_free (new_name);
 	}
+	g_regex_unref (re);
 	data->new_names_list = g_list_reverse (data->new_names_list);
 
 	if (error != NULL) {
