@@ -24,6 +24,10 @@
 #include <gtk/gtk.h>
 #include <clutter/clutter.h>
 #include <clutter-gtk/clutter-gtk.h>
+#if HAVE_GSTREAMER
+#include <gst/gst.h>
+#include <extensions/gstreamer_utils/gstreamer-utils.h>
+#endif
 #include "gth-slideshow.h"
 #include "gth-transition.h"
 
@@ -32,24 +36,32 @@
 
 
 struct _GthSlideshowPrivate {
-	GthBrowser      *browser;
-	GList           *file_list; /* GthFileData */
-	gboolean         automatic;
-	gboolean         loop;
-	GList           *current;
-	GthImageLoader  *image_loader;
-	GList           *transitions; /* GthTransition */
-	int              n_transitions;
-	GthTransition   *transition;
-	ClutterTimeline *timeline;
-	ClutterActor    *image1;
-	ClutterActor    *image2;
-	guint            next_event;
-	guint            delay;
-	guint            hide_cursor_event;
-	GRand           *rand;
-	gboolean         first_show;
-	gboolean         one_loaded;
+	GthBrowser        *browser;
+	GList             *file_list; /* GthFileData */
+	gboolean           automatic;
+	gboolean           wrap_around;
+	GList             *current;
+	GthImagePreloader *preloader;
+	GList             *transitions; /* GthTransition */
+	int                n_transitions;
+	GthTransition     *transition;
+	ClutterTimeline   *timeline;
+	ClutterActor      *image1;
+	ClutterActor      *image2;
+	guint              next_event;
+	guint              delay;
+	guint              hide_cursor_event;
+	GRand             *rand;
+	gboolean           first_show;
+	gboolean           one_loaded;
+	char             **audio_files;
+	gboolean           audio_loop;
+#if HAVE_GSTREAMER
+	int                current_audio_file;
+	GstElement        *playbin;
+#endif
+	gboolean           paused;
+	gboolean           animating;
 };
 
 
@@ -74,13 +86,17 @@ _gth_slideshow_close (GthSlideshow *self)
 static void
 _gth_slideshow_load_current_image (GthSlideshow *self)
 {
+	GthFileData *requested_file;
+	GthFileData *next_file;
+	GthFileData *prev_file;
+
 	if (self->priv->next_event != 0) {
 		g_source_remove (self->priv->next_event);
 		self->priv->next_event = 0;
 	}
 
 	if (self->priv->current == NULL) {
-		if (! self->priv->one_loaded || ! self->priv->loop) {
+		if (! self->priv->one_loaded || ! self->priv->wrap_around) {
 			_gth_slideshow_close (self);
 			return;
 		}
@@ -91,8 +107,19 @@ _gth_slideshow_load_current_image (GthSlideshow *self)
 			self->priv->current = g_list_last (self->priv->file_list);
 	}
 
-	gth_image_loader_set_file_data (GTH_IMAGE_LOADER (self->priv->image_loader), (GthFileData *) self->priv->current->data);
-	gth_image_loader_load (GTH_IMAGE_LOADER (self->priv->image_loader));
+	requested_file = (GthFileData *) self->priv->current->data;
+	if (self->priv->current->next != NULL)
+		next_file = (GthFileData *) self->priv->current->next->data;
+	else
+		next_file = NULL;
+	if (self->priv->current->prev != NULL)
+		prev_file = (GthFileData *) self->priv->current->prev->data;
+	else
+		prev_file = NULL;
+	gth_image_preloader_load (self->priv->preloader,
+				  requested_file,
+				  next_file,
+				  prev_file);
 }
 
 
@@ -177,6 +204,7 @@ _gth_slideshow_reset_textures_position (GthSlideshow *self)
 static void
 _gth_slideshow_animation_completed (GthSlideshow *self)
 {
+	self->priv->animating = FALSE;
 	if (clutter_timeline_get_direction (self->priv->timeline) == CLUTTER_TIMELINE_FORWARD)
 		_gth_slideshow_swap_current_and_next (self);
 	_gth_slideshow_reset_textures_position (self);
@@ -191,6 +219,9 @@ _gth_slideshow_load_next_image (GthSlideshow *self)
 		_gth_slideshow_animation_completed (self);
 	}
 
+	if (self->priv->paused)
+		return;
+
 	self->priv->current = self->priv->current->next;
 	clutter_timeline_set_direction (self->priv->timeline, CLUTTER_TIMELINE_FORWARD);
 	_gth_slideshow_load_current_image (self);
@@ -256,6 +287,7 @@ static void
 animation_started_cb (ClutterTimeline *timeline,
 		      GthSlideshow    *self)
 {
+	self->priv->animating = TRUE;
 	self->first_frame = TRUE;
 }
 
@@ -274,16 +306,18 @@ _gth_slideshow_get_transition (GthSlideshow *self)
 
 
 static void
-image_loader_ready_cb (GthImageLoader *image_loader,
-		       GError         *error,
-		       GthSlideshow   *self)
+image_preloader_requested_ready_cb (GthImagePreloader *preloader,
+				    GError            *error,
+				    gpointer           user_data)
 {
-	GdkPixbuf    *pixbuf;
-	GdkPixbuf    *image;
-	ClutterActor *texture;
-	int           pixbuf_w, pixbuf_h;
-	float         stage_w, stage_h;
-	int           pixbuf_x, pixbuf_y;
+	GthSlideshow   *self = user_data;
+	GthImageLoader *image_loader;
+	GdkPixbuf      *pixbuf;
+	GdkPixbuf      *image;
+	ClutterActor   *texture;
+	int             pixbuf_w, pixbuf_h;
+	float           stage_w, stage_h;
+	int             pixbuf_x, pixbuf_y;
 
 	if (error != NULL) {
 		g_clear_error (&error);
@@ -297,7 +331,18 @@ image_loader_ready_cb (GthImageLoader *image_loader,
 	if ((stage_w == 0) || (stage_h == 0))
 		return;
 
-	pixbuf = gth_image_loader_get_pixbuf (GTH_IMAGE_LOADER (image_loader));
+	image_loader = gth_image_preloader_get_loader (self->priv->preloader, (GthFileData *) self->priv->current->data);
+	if (image_loader == NULL) {
+		_gth_slideshow_load_next_image (self);
+		return;
+	}
+
+	pixbuf = gth_image_loader_get_pixbuf (image_loader);
+	if (image_loader == NULL) {
+		_gth_slideshow_load_next_image (self);
+		return;
+	}
+
 	image = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (pixbuf),
 				FALSE,
 				gdk_pixbuf_get_bits_per_sample (pixbuf),
@@ -355,14 +400,20 @@ gth_slideshow_init (GthSlideshow *self)
 	self->priv->next_event = 0;
 	self->priv->delay = DEFAULT_DELAY;
 	self->priv->automatic = FALSE;
-	self->priv->loop = FALSE;
+	self->priv->wrap_around = FALSE;
 	self->priv->transitions = NULL;
 	self->priv->n_transitions = 0;
 	self->priv->rand = g_rand_new ();
 	self->priv->first_show = TRUE;
+	self->priv->audio_files = NULL;
+	self->priv->paused = FALSE;
+	self->priv->animating = FALSE;
 
-	self->priv->image_loader = gth_image_loader_new (FALSE);
-	g_signal_connect (self->priv->image_loader, "ready", G_CALLBACK (image_loader_ready_cb), self);
+	self->priv->preloader = gth_image_preloader_new ();
+	g_signal_connect (self->priv->preloader,
+			  "requested_ready",
+			  G_CALLBACK (image_preloader_requested_ready_cb),
+			  self);
 }
 
 
@@ -378,10 +429,19 @@ gth_slideshow_finalize (GObject *object)
 
 	_g_object_list_unref (self->priv->file_list);
 	_g_object_unref (self->priv->browser);
-	_g_object_unref (self->priv->image_loader);
+	_g_object_unref (self->priv->preloader);
 	_g_object_unref (self->priv->timeline);
 	_g_object_list_unref (self->priv->transitions);
 	g_rand_free (self->priv->rand);
+	g_strfreev (self->priv->audio_files);
+
+#if HAVE_GSTREAMER
+	if (self->priv->playbin != NULL) {
+		gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
+		gst_object_unref (GST_OBJECT (self->priv->playbin));
+		self->priv->playbin = NULL;
+	}
+#endif
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -442,6 +502,30 @@ hide_cursor_cb (gpointer data)
 }
 
 
+static void
+_gth_slideshow_toggle_pause (GthSlideshow *self)
+{
+	self->priv->paused = ! self->priv->paused;
+	if (self->priv->paused) {
+		if (self->priv->animating) {
+			clutter_timeline_pause (self->priv->timeline);
+			_gth_slideshow_animation_completed (self);
+		}
+#if HAVE_GSTREAMER
+		if (self->priv->playbin != NULL)
+			gst_element_set_state (self->priv->playbin, GST_STATE_PAUSED);
+#endif
+	}
+	else { /* resume */
+		_gth_slideshow_load_next_image (self);
+#if HAVE_GSTREAMER
+		if (self->priv->playbin != NULL)
+			gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
+#endif
+	}
+}
+
+
 static void
 stage_input_cb (ClutterStage *stage,
 	        ClutterEvent *event,
@@ -460,10 +544,16 @@ stage_input_cb (ClutterStage *stage,
 			break;
 
 		case CLUTTER_space:
+			_gth_slideshow_toggle_pause (self);
+			break;
+
+		case CLUTTER_Up:
+		case CLUTTER_Right:
 			_gth_slideshow_load_next_image (self);
 			break;
 
-		case CLUTTER_BackSpace:
+		case CLUTTER_Down:
+		case CLUTTER_Left:
 			_gth_slideshow_load_prev_image (self);
 			break;
 		}
@@ -471,6 +561,29 @@ stage_input_cb (ClutterStage *stage,
 }
 
 
+#if HAVE_GSTREAMER
+static void
+bus_message_cb (GstBus     *bus,
+                GstMessage *message,
+                gpointer    user_data)
+{
+	GthSlideshow *self = user_data;
+
+	if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_EOS) {
+		self->priv->current_audio_file++;
+		if ((self->priv->audio_files[self->priv->current_audio_file] == NULL)
+		    && self->priv->audio_loop)
+		{
+			self->priv->current_audio_file = 0;
+		}
+		gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
+		g_object_set (G_OBJECT (self->priv->playbin), "uri", self->priv->audio_files[self->priv->current_audio_file], NULL);
+		gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
+	}
+}
+#endif
+
+
 static void
 gth_slideshow_show_cb (GtkWidget    *widget,
 		       GthSlideshow *self)
@@ -478,6 +591,26 @@ gth_slideshow_show_cb (GtkWidget    *widget,
 	if (! self->priv->first_show)
 		return;
 
+#if HAVE_GSTREAMER
+	if (gstreamer_init()) {
+		if ((self->priv->audio_files != NULL) && (self->priv->audio_files[0] != NULL)) {
+			self->priv->current_audio_file = 0;
+			if (self->priv->playbin == NULL) {
+				GstBus *bus;
+
+				self->priv->playbin = gst_element_factory_make ("playbin", "playbin");
+				bus = gst_pipeline_get_bus (GST_PIPELINE (self->priv->playbin));
+				gst_bus_add_signal_watch (bus);
+				g_signal_connect (bus, "message", G_CALLBACK (bus_message_cb), self);
+			}
+			else
+				gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
+			g_object_set (G_OBJECT (self->priv->playbin), "uri", self->priv->audio_files[self->priv->current_audio_file], NULL);
+			gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
+		}
+	}
+#endif
+
 	_gth_slideshow_load_current_image (self);
 	self->priv->first_show = FALSE;
 }
@@ -556,10 +689,10 @@ gth_slideshow_set_automatic (GthSlideshow *self,
 
 
 void
-gth_slideshow_set_loop (GthSlideshow *self,
-			gboolean      loop)
+gth_slideshow_set_wrap_around (GthSlideshow *self,
+			       gboolean      wrap_around)
 {
-	self->priv->loop = loop;
+	self->priv->wrap_around = wrap_around;
 }
 
 
@@ -571,3 +704,12 @@ gth_slideshow_set_transitions (GthSlideshow *self,
 	self->priv->transitions = _g_object_list_ref (transitions);
 	self->priv->n_transitions = g_list_length (self->priv->transitions);
 }
+
+
+void
+gth_slideshow_set_playlist (GthSlideshow  *self,
+			    char         **files)
+{
+	self->priv->audio_files = g_strdupv (files);
+	self->priv->audio_loop = TRUE;
+}
