@@ -40,6 +40,7 @@
 #define UPDATE_THUMBNAILS_TIMEOUT 250
 #define N_LOOKAHEAD 50
 #define EMPTY (N_("(Empty)"))
+#define THUMBNAIL_BORDER (8 * 2)
 
 typedef enum {
 	GTH_FILE_LIST_OP_TYPE_SET_FILES,
@@ -253,8 +254,8 @@ gth_file_list_class_init (GthFileListClass *class)
 	GObjectClass *object_class;
 
 	parent_class = g_type_class_peek_parent (class);
-	object_class = (GObjectClass*) class;
 
+	object_class = (GObjectClass*) class;
 	object_class->finalize = gth_file_list_finalize;
 }
 
@@ -483,7 +484,7 @@ gth_file_list_construct (GthFileList     *file_list,
 
 	/* thumbnail loader */
 
-	file_list->priv->thumb_loader = gth_thumb_loader_new (file_list->priv->thumb_size, file_list->priv->thumb_size);
+	file_list->priv->thumb_loader = gth_thumb_loader_new (file_list->priv->thumb_size);
 	g_signal_connect (G_OBJECT (file_list->priv->thumb_loader),
 			  "ready",
 			  G_CALLBACK (thumb_loader_ready_cb),
@@ -528,7 +529,7 @@ gth_file_list_construct (GthFileList     *file_list,
 	file_list->priv->view = gth_icon_view_new_with_model (GTK_TREE_MODEL (model));
 	g_object_unref (model);
 
-	if (file_list->priv->type == GTH_FILE_LIST_TYPE_SELECTOR)
+	if ((file_list->priv->type == GTH_FILE_LIST_TYPE_SELECTOR) || (file_list->priv->type == GTH_FILE_LIST_TYPE_NO_SELECTION))
 		gth_file_selection_set_selection_mode (GTH_FILE_SELECTION (file_list->priv->view), GTK_SELECTION_NONE);
 	else
 		gth_file_selection_set_selection_mode (GTH_FILE_SELECTION (file_list->priv->view), GTK_SELECTION_MULTIPLE);
@@ -560,7 +561,7 @@ gth_file_list_construct (GthFileList     *file_list,
 					"active", GTH_FILE_STORE_CHECKED_COLUMN,
 					NULL);
 	g_object_set (file_list->priv->checkbox_renderer,
-		      "visible", (file_list->priv->type != GTH_FILE_LIST_TYPE_NORMAL),
+		      "visible", ((file_list->priv->type == GTH_FILE_LIST_TYPE_BROWSER) || (file_list->priv->type == GTH_FILE_LIST_TYPE_SELECTOR)),
 		      NULL);
 	g_signal_connect (file_list->priv->checkbox_renderer,
 			  "toggled",
@@ -598,10 +599,11 @@ gth_file_list_construct (GthFileList     *file_list,
 
 	file_list->priv->text_renderer = renderer = gtk_cell_renderer_text_new ();
 	g_object_set (G_OBJECT (renderer),
-		      "ellipsize", PANGO_ELLIPSIZE_END,
+		      /*"ellipsize", PANGO_ELLIPSIZE_NONE,*/
 		      "alignment", PANGO_ALIGN_CENTER,
-		      "width", file_list->priv->thumb_size + (8 * 2) /* FIXME: remove the numbers */,
-		      /*"single-paragraph-mode", TRUE,*/
+		      "width", file_list->priv->thumb_size + THUMBNAIL_BORDER,
+		      "wrap-mode", PANGO_WRAP_WORD_CHAR,
+		      "wrap-width", file_list->priv->thumb_size + THUMBNAIL_BORDER,
 		      NULL);
 
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (file_list->priv->view), renderer, FALSE);
@@ -713,6 +715,13 @@ gth_file_list_cancel (GthFileList *file_list,
 }
 
 
+GthThumbLoader *
+gth_file_list_get_thumb_loader (GthFileList *file_list)
+{
+	return file_list->priv->thumb_loader;
+}
+
+
 static void
 gfl_clear_list (GthFileList *file_list,
 		const char  *message)
@@ -1115,16 +1124,17 @@ gth_file_list_set_thumb_size (GthFileList *file_list,
 			      int          size)
 {
 	file_list->priv->thumb_size = size;
-	gth_thumb_loader_set_thumb_size (file_list->priv->thumb_loader, size, size);
+	gth_thumb_loader_set_requested_size (file_list->priv->thumb_loader, size);
 
 	gth_icon_cache_free (file_list->priv->icon_cache);
 	file_list->priv->icon_cache = gth_icon_cache_new (gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (file_list))), size / 2);
 
 	g_object_set (file_list->priv->thumbnail_renderer,
-		      "size", size,
+		      "size", file_list->priv->thumb_size,
 		      NULL);
 	g_object_set (file_list->priv->text_renderer,
-		      "width", size + (8 * 2),
+		      "width", file_list->priv->thumb_size + THUMBNAIL_BORDER,
+		      "wrap-width", file_list->priv->thumb_size + THUMBNAIL_BORDER,
 		      NULL);
 }
 
@@ -1273,9 +1283,13 @@ can_create_file_thumbnail (GthFileData *file_data,
 			   GTimeVal    *current_time,
 			   gboolean    *young_file_found)
 {
+	time_t   time_diff;
 	gboolean young_file;
 
-	young_file = (current_time->tv_sec - gth_file_data_get_mtime (file_data)) <= 1;
+	/* Check for files that are exactly 0 or 1 seconds old; they may still be changing. */
+	time_diff = current_time->tv_sec - gth_file_data_get_mtime (file_data);
+	young_file = (time_diff <= 1) && (time_diff >= 0);
+
 	if (young_file)
 		*young_file_found = TRUE;
 
